version='0.0.1'
package_name='fakr'
repo_url='http://github.com/l-x/{}'.format(package_name)
download_url="{}/archive/{}.tar.gz".format(repo_url, version)
author = 'l-x',
author_email = 'l-x+github@mailbox.org',